"""
/**************************************************************
 LZSS.C -- A Data Compression Program
***************************************************************
    4/6/1989 Haruhiko Okumura
    Use, distribute, and modify this program freely.
    Please send me your improved versions.
        PC-VAN      SCIENCE
        NIFTY-Serve PAF01022
        CompuServe  74050,1022

**************************************************************/
/*
 *  lzss.c - Package for decompressing lzss compressed objects
 *
 *  Copyright (c) 2003 Apple Computer, Inc.
 *
 *  DRI: Josh de Cesare
 */
"""
from array import array
import struct

N = 4096
F = 18
THRESHOLD = 2
NIL = N


def decompress_lzss(str):
    if str[:8] != "complzss":
        print("decompress_lzss: complzss magic missing")
        return
    decompsize = struct.unpack(">L", str[12:16])[0]
    text_buf = array("B", " " * (N + F - 1))
    src = array("B", str[0x180:])
    srclen = len(src)
    dst = array("B", " " * decompsize)
    r = N - F
    srcidx, dstidx, flags, c = 0, 0, 0, 0

    while True:
        flags >>= 1
        if ((flags & 0x100) == 0):
            if (srcidx >= srclen):
                break
            c = src[srcidx]
            srcidx += 1
            flags = c | 0xFF00

        if (flags & 1):
            if (srcidx >= srclen):
                break
            c = src[srcidx]
            srcidx += 1
            dst[dstidx] = c
            dstidx += 1
            text_buf[r] = c
            r += 1
            r &= (N - 1)
        else:
            if (srcidx >= srclen):
                break
            i = src[srcidx]
            srcidx += 1
            if (srcidx >= srclen):
                break
            j = src[srcidx]
            srcidx += 1
            i |= ((j & 0xF0) << 4)
            j = (j & 0x0F) + THRESHOLD
            for k in range(j + 1):
                c = text_buf[(i + k) & (N - 1)]
                dst[dstidx] = c
                dstidx += 1
                text_buf[r] = c
                r += 1
                r &= (N - 1)
    return dst.tostring()



#!/usr/bin/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import struct
import time
import sys
from tempfile import mkstemp
from pymobiledevice.lockdown import LockdownClient
from optparse import OptionParser

"""
struct pcap_hdr_s {
        guint32 magic_number;   /* magic number */
        guint16 version_major;  /* major version number */
        guint16 version_minor;  /* minor version number */
        gint32  thiszone;       /* GMT to local correction */
        guint32 sigfigs;        /* accuracy of timestamps */
        guint32 snaplen;        /* max length of captured packets, in octets */
        guint32 network;        /* data link type */
} pcap_hdr_t;
typedef struct pcaprec_hdr_s {
        guint32 ts_sec;         /* timestamp seconds */
        guint32 ts_usec;        /* timestamp microseconds */
        guint32 incl_len;       /* number of octets of packet saved in file */
        guint32 orig_len;       /* actual length of packet */
} pcaprec_hdr_t;
"""
LINKTYPE_ETHERNET = 1
LINKTYPE_RAW = 101


class PcapOut(object):

    def __init__(self, pipename=r'test.pcap'):
        self.pipe = open(pipename, 'wb')
        self.pipe.write(struct.pack("<LHHLLLL", 0xa1b2c3d4,
                                    2, 4, 0, 0, 65535, LINKTYPE_ETHERNET))

    def __del__(self):
        self.pipe.close()

    def writePacket(self, packet):
        t = time.time()
        # TODO check milisecond conversion
        pkthdr = struct.pack('<LLLL', int(t), int(
            t * 1000000 % 1000000), len(packet), len(packet))
        data = pkthdr + packet
        l = self.pipe.write(data)
        self.pipe.flush()
        return True


class Win32Pipe(object):

    def __init__(self, pipename=r'\\.\pipe\wireshark'):
        self.pipe = win32pipe.CreateNamedPipe(pipename,
                                              win32pipe.PIPE_ACCESS_OUTBOUND,
                                              win32pipe.PIPE_TYPE_MESSAGE | win32pipe.PIPE_WAIT,
                                              1, 65536, 65536,
                                              300,
                                              None)
        print("Connect wireshark to %s" % pipename)
        win32pipe.ConnectNamedPipe(self.pipe, None)
        win32file.WriteFile(self.pipe, struct.pack(
            "<LHHLLLL", 0xa1b2c3d4, 2, 4, 0, 0, 65535, LINKTYPE_ETHERNET))

    def writePacket(self, packet):
        t = time.time()
        pkthdr = struct.pack("<LLLL", int(t), int(
            t * 1000000 % 1000000), len(packet), len(packet))
        errCode, nBytesWritten = win32file.WriteFile(
            self.pipe, pkthdr + packet)
        return errCode == 0

if __name__ == "__main__":

    if sys.platform == "darwin":
        print("Why not use rvictl ?")

    parser = OptionParser(usage="%prog")
    parser.add_option("-o", "--output", dest="output", default=False,
                      help="Output location", type="string")

    (options, args) = parser.parse_args()
    if sys.platform == "win32":
        import win32pipe
        import win32file
        output = Win32Pipe()

    else:
        if options.output:
            path = options.output
        else:
            _, path = mkstemp(prefix="device_dump_", suffix=".pcap", dir=".")
        print("Recording data to: %s" % path)
        output = PcapOut(path)

    lockdown = LockdownClient()
    pcap = lockdown.startService("com.apple.pcapd")

    while True:
        d = pcap.recvPlist()
        if not d:
            break
        data = d.data
        hdrsize, xxx, packet_size = struct.unpack(">LBL", data[:9])
        flags1, flags2, offset_to_ip_data, zero = struct.unpack(">LLLL", data[
                                                                9:0x19])

        assert hdrsize >= 0x19
        interfacetype = data[0x19:hdrsize].strip("\x00")
        t = time.time()
        print(interfacetype, packet_size, t)

        packet = data[hdrsize:]
        assert packet_size == len(packet)

        if offset_to_ip_data == 0:
            # add fake ethernet header for pdp packets
            packet = "\xBE\xEF" * 6 + "\x08\x00" + packet
        if not output.writePacket(packet):
            break



#!/usr/bin/env python

"""
How to create a CA certificate with Python.

WARNING: This sample only demonstrates how to use the objects and methods,
         not how to create a safe and correct certificate.

Copyright (c) 2004 Open Source Applications Foundation.
Author: Heikki Toivonen
"""

from M2Crypto import RSA, X509, EVP, m2, Rand, Err, BIO
from M2Crypto.RSA import load_pub_key_bio
import base64


def der_length(length):
    # DER encoding of a length
    if length < 128:
        return chr(length)
    prefix = 0x80
    result = ''
    while length > 0:
        result = chr(length & 0xff) + result
        length >>= 8
        prefix += 1
    return chr(prefix) + result


def convertPKCS1toPKCS8pubKey(data):
    # https://mail.python.org/pipermail/python-list/2013-January/638754.html
    res = data.split('\n')
    res = '\0' + base64.decodestring("".join(res[1:-2]))
    res = '\x30\x0d\x06\x09\x2a\x86\x48\x86\xf7\x0d\x01\x01\x01\x05\x00\x03' + \
        der_length(len(res)) + res
    res = '\x30' + der_length(len(res)) + res
    res = '-----BEGIN PUBLIC KEY-----\n' + \
        base64.encodestring(res) + '-----END PUBLIC KEY-----'
    return res

# XXX Do I actually need more keys?
# XXX Check return values from functions


def generateRSAKey():
    return RSA.gen_key(2048, m2.RSA_F4)


def makePKey(key):
    pkey = EVP.PKey()
    pkey.assign_rsa(key)
    return pkey


def makeRequest(pkey, cn):
    req = X509.Request()
    req.set_version(2)
    req.set_pubkey(pkey)
    name = X509.X509_Name()
    name.CN = cn
    req.set_subject_name(name)
    ext1 = X509.new_extension('subjectAltName', 'DNS:foobar.example.com')
    ext2 = X509.new_extension('nsComment', 'Hello there')
    extstack = X509.X509_Extension_Stack()
    extstack.push(ext1)
    extstack.push(ext2)

    assert(extstack[1].get_name() == 'nsComment')

    req.add_extensions(extstack)
    #req.sign(pkey, 'sha1')
    return req


def makeCert(req, caPkey):
    pkey = req.get_pubkey()
    #woop = makePKey(generateRSAKey())
    # if not req.verify(woop.pkey):
    if not req.verify(pkey):
        # XXX What error object should I use?
        raise ValueError('Error verifying request')
    sub = req.get_subject()
    # If this were a real certificate request, you would display
    # all the relevant data from the request and ask a human operator
    # if you were sure. Now we just create the certificate blindly based
    # on the request.
    cert = X509.X509()
    # We know we are making CA cert now...
    # Serial defaults to 0.
    cert.set_serial_number(1)
    cert.set_version(2)
    cert.set_subject(sub)
    issuer = X509.X509_Name()
    issuer.CN = 'The Issuer Monkey'
    issuer.O = 'The Organization Otherwise Known as My CA, Inc.'
    cert.set_issuer(issuer)
    cert.set_pubkey(pkey)
    notBefore = m2.x509_get_not_before(cert.x509)
    notAfter = m2.x509_get_not_after(cert.x509)
    m2.x509_gmtime_adj(notBefore, 0)
    days = 30
    m2.x509_gmtime_adj(notAfter, 60 * 60 * 24 * days)
    cert.add_ext(
        X509.new_extension('subjectAltName', 'DNS:foobar.example.com'))
    ext = X509.new_extension('nsComment', 'M2Crypto generated certificate')
    ext.set_critical(0)  # Defaults to non-critical, but we can also set it
    cert.add_ext(ext)
    cert.sign(caPkey, 'sha1')

    assert(cert.get_ext('subjectAltName').get_name() == 'subjectAltName')
    assert(cert.get_ext_at(0).get_name() == 'subjectAltName')
    assert(cert.get_ext_at(0).get_value() == 'DNS:foobar.example.com')

    return cert


def ca():
    key = generateRSAKey()
    pkey = makePKey(key)
    req = makeRequest(pkey)
    cert = makeCert(req, pkey)
    return (cert, pkey)


def ca_do_everything(DevicePublicKey):
    rsa = generateRSAKey()
    privateKey = makePKey(rsa)
    req = makeRequest(privateKey, "The Issuer Monkey")
    cert = makeCert(req, privateKey)
    rsa2 = load_pub_key_bio(BIO.MemoryBuffer(
        convertPKCS1toPKCS8pubKey(DevicePublicKey)))
    pkey2 = EVP.PKey()
    pkey2.assign_rsa(rsa2)
    req = makeRequest(pkey2, "Device")
    cert2 = makeCert(req, privateKey)
    return cert.as_pem(), privateKey.as_pem(None), cert2.as_pem()

if __name__ == '__main__':
    rsa = generateRSAKey()
    pkey = makePKey(rsa)
    print(pkey.as_pem(None))
    req = makeRequest(pkey, "The Issuer Monkey")
    # print req.as_text()
    cert = makeCert(req, pkey)
    print(cert.as_text())
    cert.save_pem('my_ca_cert.pem')
    rsa.save_key('my_key.pem', None)



#!/usr/bin/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#


from pymobiledevice.lockdown import LockdownClient
from datetime import datetime
from pymobiledevice.util import getHomePath
from pymobiledevice.util import hexdump
from sys import exit
from optparse import OptionParser

import re


class Syslog(object):

    def __init__(self, lockdown=None):
        if lockdown:
            self.lockdown = lockdown
        else:
            self.lockdown = LockdownClient()
        self.c = self.lockdown.startService("com.apple.syslog_relay")
        if self.c:
            self.c.send("watch")
        else:
            sys.exit(1)

    def watch(self, procName=None, logFile=None):

        while True:
            d = self.c.recv(4096)

            if not d:
                break

            if procName:
                procFilter = re.compile(procName, re.IGNORECASE)
                if len(d.split(" ")) > 4 and not procFilter.search(d):
                    continue

            print(d.strip(b"\n\x00\x00"))

            if logFile:
                with open(logFile, 'a') as f:
                    f.write(d.replace("\x00", ""))

if __name__ == "__main__":
    parser = OptionParser(usage="%prog")
    parser.add_option("-p", "--process", dest="procName", default=False,
                      help="Show process log only", type="string")
    parser.add_option("-o", "--logfile", dest="logFile", default=False,
                      help="Write Logs into specified file", type="string")
    (options, args) = parser.parse_args()

    try:
        while True:
            try:
                syslog = Syslog()
                syslog.watch(procName=options.procName,
                             logFile=options.logFile)
            except KeyboardInterrupt:
                print("KeyboardInterrupt caught")
                raise
            else:
                pass

    except (KeyboardInterrupt, SystemExit):
        exit()



#!/usr/bin/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

from pymobiledevice.lockdown import LockdownClient
from pymobiledevice.util.cpio import CpioArchive
from pymobiledevice.util import MultipleOption
import zlib
import gzip
from pprint import pprint
from tempfile import mkstemp
from optparse import OptionParser
from io import BytesIO
import os

SRCFILES = """Baseband
CrashReporter
MobileAsset
VARFS
HFSMeta
Lockdown
MobileBackup
MobileDelete
MobileInstallation
MobileNotes
Network
UserDatabases
WiFi
WirelessAutomation
NANDDebugInfo
SystemConfiguration
Ubiquity
tmp
WirelessAutomation"""


class DeviceVersionNotSupported(Exception):
    pass


class FileRelay(object):

    def __init__(self, lockdown=None, serviceName="com.apple.mobile.file_relay"):
        if lockdown:
            self.lockdown = lockdown
        else:
            self.lockdown = LockdownClient()

        ProductVersion = self.lockdown.getValue("", "ProductVersion")

        if ProductVersion[0] >= "8":
            raise DeviceVersionNotSupported

        self.service = self.lockdown.startService(serviceName)
        self.packet_num = 0

    def stop_session(self):
        print("Disconecting...")
        self.service.close()

    def request_sources(self, sources=["UserDatabases"]):
        self.service.sendPlist({"Sources": sources})
        while 1:
            res = self.service.recvPlist()
            if res:
                s = res.get("Status")
                if s == "Acknowledged":
                    z = ""
                    while True:
                        x = self.service.recv()
                        if not x:
                            break
                        z += x
                    return z
                else:
                    print(res.get("Error"))
                    break
        return None

if __name__ == "__main__":

    parser = OptionParser(option_class=MultipleOption, usage="%prog")
    parser.add_option("-s", "--sources",
                      action="extend",
                      dest="sources",
                      metavar='SOURCES',
                      choices=SRCFILES.split("\n"),
                      help="comma separated list of file relay source to dump")
    parser.add_option("-e", "--extract", dest="extractpath", default=False,
                      help="Extract archive to specified location", type="string")
    parser.add_option("-o", "--output", dest="outputfile", default=False,
                      help="Output location", type="string")

    (options, args) = parser.parse_args()

    sources = []
    if options.sources:
        sources = options.sources
    else:
        sources = ["UserDatabases"]
    print("Downloading: %s" % ''.join([str(item) + " " for item in sources]))

    fc = None
    try:
        fc = FileRelay()
    except:
        print(
            "Device with product vertion >= 8.0 does not allow access to fileRelay service")
        exit()

    data = fc.request_sources(sources)

    if data:
        if options.outputfile:
            path = options.outputfile
        else:
            _, path = mkstemp(prefix="fileRelay_dump_", suffix=".gz", dir=".")

        open(path, 'wb').write(data)
        print("Data saved to:  %s " % path)

    if options.extractpath:
        with open(path, 'r') as f:
            gz = gzip.GzipFile(mode='rb', fileobj=f)
            cpio = CpioArchive(fileobj=BytesIO(gz.read()))
            cpio.extract_files(files=None, outpath=options.extractpath)



#!/usr/bin/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#


from pymobiledevice.lockdown import LockdownClient
import struct
import plistlib
from pprint import pprint
import os
import datetime
from pymobiledevice.afc import AFCClient
from pymobiledevice.util import makedirs

#
# Fix plistlib.py line 364
#     def asBase64(self, maxlinelength=76):
#        if self.data != None:
#            return _encodeBase64(self.data, maxlinelength)
#        return ""
#
#


MOBILEBACKUP_E_SUCCESS = 0
MOBILEBACKUP_E_INVALID_ARG = -1
MOBILEBACKUP_E_PLIST_ERROR = -2
MOBILEBACKUP_E_MUX_ERROR = -3
MOBILEBACKUP_E_BAD_VERSION = -4
MOBILEBACKUP_E_REPLY_NOT_OK = -5
MOBILEBACKUP_E_UNKNOWN_ERROR = -256

DEVICE_LINK_FILE_STATUS_NONE = 0
DEVICE_LINK_FILE_STATUS_HUNK = 1
DEVICE_LINK_FILE_STATUS_LAST_HUNK = 2


class DeviceVersionNotSupported(Exception):

    def __str__(self):
        return "Device version not supported, please use mobilebackup2"


class MobileBackup(object):

    def __init__(self, lockdown=None):
        if lockdown:
            self.lockdown = lockdown
        else:
            self.lockdown = LockdownClient()

        ProductVersion = self.lockdown.getValue("", "ProductVersion")
        if ProductVersion[0] >= "5":
            raise DeviceVersionNotSupported

        self.service = self.lockdown.startService("com.apple.mobilebackup")
        self.udid = self.lockdown.udid
        DLMessageVersionExchange = self.service.recvPlist()
        version_major = DLMessageVersionExchange[1]
        self.service.sendPlist(
            ["DLMessageVersionExchange", "DLVersionsOk", version_major])
        DLMessageDeviceReady = self.service.recvPlist()
        if DLMessageDeviceReady and DLMessageDeviceReady[0] == "DLMessageDeviceReady":
            print("Got DLMessageDeviceReady")

    def check_filename(self, name):
        if name.find("../") != -1:
            raise Exception("HAX, sneaky dots in path %s" % name)
        if not name.startswith(self.backupPath):
            if name.startswith(self.udid):
                name = os.path.join(self.backupPath, name)
                return name
            name = os.path.join(self.backupPath, self.udid, name)
            return name
        return name

    def read_file(self, filename):
        filename = self.check_filename(filename)
        if os.path.isfile(filename):
            with open(filename, 'rb') as f:
                data = f.read()
                f.close()
                return data
        return None

    def write_file(self, filename, data):
        filename = self.check_filename(filename)
        with open(filename, 'wb') as f:
            f.write(data)
            f.close()

    def create_info_plist(self):
        root_node = self.lockdown.allValues
        # print pprint(root_node)
        info = {"BuildVersion": root_node.get("BuildVersion") or "",
                "DeviceName":  root_node.get("DeviceName") or "",
                "Display Name": root_node.get("DeviceName") or "",
                "GUID": "---",
                "ProductType": root_node.get("ProductType") or "",
                "ProductVersion": root_node.get("ProductVersion") or "",
                "Serial Number": root_node.get("SerialNumber") or "",
                "Unique Identifier": self.udid.upper(),
                "Target Identifier": self.udid,
                "Target Type": "Device",
                "iTunes Version": "10.0.1"
                }
        info["ICCID"] = root_node.get("IntegratedCircuitCardIdentity") or ""
        info["IMEI"] = root_node.get(
            "InternationalMobileEquipmentIdentity") or ""
        info["Last Backup Date"] = datetime.datetime.now()

        afc = AFCClient(self.lockdown)
        iTunesFilesDict = {}
        iTunesFiles = afc.read_directory("/iTunes_Control/iTunes/")

        for i in iTunesFiles:
            data = afc.get_file_contents("/iTunes_Control/iTunes/" + i)
            if data:
                iTunesFilesDict[i] = plistlib.Data(data)
        info["iTunesFiles"] = iTunesFilesDict

        iBooksData2 = afc.get_file_contents("/Books/iBooksData2.plist")
        if iBooksData2:
            info["iBooks Data 2"] = plistlib.Data(iBooksData2)

        info["iTunes Settings"] = self.lockdown.getValue("com.apple.iTunes")
        print("Creating %s" % os.path.join(self.udid, "Info.plist"))
        self.write_file(os.path.join(self.udid, "Info.plist"),
                        plistlib.dumps(info))

    def ping(self, message):
        self.service.sendPlist(["DLMessagePing", message])
        print("ping response", self.service.recvPlist())

    def device_link_service_send_process_message(self, msg):
        return self.service.sendPlist(["DLMessageProcessMessage", msg])

    def device_link_service_receive_process_message(self):
        req = self.service.recvPlist()
        if req:
            assert req[0] == "DLMessageProcessMessage"
            return req[1]

    def send_file_received(self):
        return self.device_link_service_send_process_message({"BackupMessageTypeKey": "kBackupMessageBackupFileReceived"})

    def request_backup(self):
        req = {"BackupComputerBasePathKey": "/",
               "BackupMessageTypeKey": "BackupMessageBackupRequest",
               "BackupProtocolVersion": "1.6"
               }
        self.create_info_plist()
        self.device_link_service_send_process_message(req)
        res = self.device_link_service_receive_process_message()
        if not res:
            return
        if res["BackupMessageTypeKey"] != "BackupMessageBackupReplyOK":
            print(res)
            return
        self.device_link_service_send_process_message(res)

        filedata = ""
        f = None
        outpath = None
        while True:
            res = self.service.recvPlist()
            if not res or res[0] != "DLSendFile":
                if res[0] == "DLMessageProcessMessage":
                    if res[1].get("BackupMessageTypeKey") == "BackupMessageBackupFinished":
                        print("Backup finished OK !")
                        # TODO BackupFilesToDeleteKey
                        plistlib.writePlist(
                            res[1]["BackupManifestKey"], self.check_filename("Manifest.plist"))
                break
            data = res[1].data
            info = res[2]
            if not f:
                outpath = self.check_filename(info.get("DLFileDest"))
                print(info["DLFileAttributesKey"][
                      "Filename"], info.get("DLFileDest"))
                f = open(outpath + ".mddata", "wb")
            f.write(data)
            if info.get("DLFileStatusKey") == DEVICE_LINK_FILE_STATUS_LAST_HUNK:
                self.send_file_received()
                f.close()
                if not info.get("BackupManifestKey", False):
                    plistlib.writePlist(
                        info.get("BackupFileInfo"), outpath + ".mdinfo")
                f = None

if __name__ == "__main__":
    mb = MobileBackup()
    mb.request_backup()



#!/usr/bin/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

from pymobiledevice.lockdown import LockdownClient
from optparse import OptionParser
import os
from pymobiledevice.afc import AFCClient


client_options = {
    "SkipUninstall": False,
    "ApplicationSINF": False,
    "iTunesMetadata": False,
    "ReturnAttributes": False
}


class installation_proxy(object):

    def __init__(self, lockdown=None):

        if lockdown:
            self.lockdown = lockdown
        else:
            self.lockdown = LockdownClient()

        self.service = self.lockdown.startService(
            "com.apple.mobile.installation_proxy")

    def watch_completion(self, handler=None, *args):
        z = self.service.recvPlist()
        while 'PercentComplete' in z:
            if not z:
                break
            completion = z.get("PercentComplete")
            if completion:
                if handler:
                    print("calling handler")
                    handler(completion, *args)
                print("%s %% Complete" % z.get("PercentComplete"))
            if z.get("Status") == "Complete":
                return z.get("Status")
            z = self.service.recvPlist()
        print(z)

    def send_cmd_for_bid(self, bundleID, cmd="Archive", options=None, handler=None, *args):
        cmd = {"Command": cmd, "ApplicationIdentifier": bundleID}
        if options:
            cmd.update(options)
        self.service.sendPlist(cmd)
        # print "%s : " % (cmd, bundleID)
        print("%s : %s\n" % (cmd, self.watch_completion(handler, *args)))

    def uninstall(self, bundleID, options=None, handler=None, *args):
        self.send_cmd_for_bid(bundleID, "Uninstall", options, handler, args)

    def install_or_upgrade(self, ipaPath, cmd="Install", options=None, handler=None, *args):
        afc = AFCClient(self.lockdown)
        afc.set_file_contents(
            "/" + os.path.basename(ipaPath), open(ipaPath, "rb").read())
        cmd = {"Command": cmd, "PackagePath": os.path.basename(ipaPath)}
        if options:
            cmd.update(options)
        self.service.sendPlist(cmd)
#         print "%s : " % (cmd, bundleID)
        print("%s : %s\n" % (cmd, self.watch_completion(handler, args)))

    def install(self, ipaPath, options=None, handler=None, *args):
        return self.install_or_upgrade(ipaPath, "Install", client_options, handler, args)

    def upgrade(self, ipaPath, options=None, handler=None, *args):
        return self.install_or_upgrade(ipaPath, "Upgrade", client_options, handler, args)

    def apps_info(self):
        self.service.sendPlist({"Command": "Lookup"})
        return self.service.recvPlist().get('LookupResult')

    def archive(self, bundleID, options=None, handler=None, *args):
        self.send_cmd_for_bid(bundleID, "Archive", options, handler, args)

    def restore_archive(self, bundleID, options=None, handler=None, *args):
        self.send_cmd_for_bid(bundleID, "Restore",
                              client_options, handler, args)

    def remove_archive(self, bundleID, options={}, handler=None, *args):
        self.send_cmd_for_bid(bundleID, "RemoveArchive",
                              options, handler, args)

    def archives_info(self):
        return self.service.sendRequest({"Command": "LookupArchive"}).get("LookupResult")

    def search_path_for_bid(self, bid):
        path = None
        for a in self.get_apps(appTypes=["User", "System"]):
            if a.get("CFBundleIdentifier") == bid:
                path = a.get("Path") + "/" + a.get("CFBundleExecutable")
        return path

    def get_apps(self, appTypes=["User"]):
        return [app for app in list(self.apps_info().values())
                if app.get("ApplicationType") in appTypes]

    def print_apps(self, appType=["User"]):
        for app in self.get_apps(appType):
            print(("%s : %s => %s" % (app.get("CFBundleDisplayName"),
                                      app.get("CFBundleIdentifier"),
                                      app.get("Path") if app.get("Path")
                                      else app.get("Container"))).encode('utf-8'))

    def get_apps_bid(self, appTypes=["User"]):
        return [app["CFBundleIdentifier"]
                for app in self.get_apps()
                if app.get("ApplicationType") in appTypes]

    def close(self):
        self.service.close()

    def __del__(self):
        self.close()


if __name__ == "__main__":
    parser = OptionParser(usage="%prog cmd <command options>")
    parser.add_option("-l", "--listapps",
                      default=False,
                      action="store_true",
                      help="List installed applications")
    parser.add_option("-i", "--install",
                      default=False,
                      action="store",
                      dest="install_ipapath",
                      metavar="FILE",
                      help="Install application on device")
    parser.add_option("-r", "--remove",
                      default=False,
                      action="store",
                      dest="remove_bundleid",
                      metavar="BUNDLE_ID",
                      help="Remove (uninstall) application from device")
    parser.add_option("-u", "--upgrade",
                      default=False,
                      action="store",
                      dest="upgrade_bundleid",
                      metavar="BUNDLE_ID",
                      help="Upgrade application on device")
    parser.add_option("-a", "--archive",
                      default=False,
                      action="store",
                      dest="archive_bundleid",
                      metavar="BUNDLE_ID",
                      help="Archive application on device")

    (options, args) = parser.parse_args()
    # FIXME We should handle all installation proxy commands
    if options.listapps:
        instpxy = installation_proxy()
        instpxy.print_apps(["User", "System"])
    elif options.install_ipapath:
        instpxy = installation_proxy()
        instpxy.install(options.install_ipapath)
    elif options.remove_bundleid:
        instpxy = installation_proxy()
        instpxy.uninstall(options.remove_bundleid)
    elif options.upgrade_bundleid:
        instpxy = installation_proxy()
        instpxy.upgrade(options.upgrade_bundleid)
    elif options.archive_bundleid:
        instpxy = installation_proxy()
        instpxy.archive(options.archive_bundleid)
    else:
        parser.error("Incorrect number of arguments")



#!/usr/bin/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#


from pymobiledevice.lockdown import LockdownClient
from pprint import pprint
import plistlib
from optparse import OptionParser

Requests = """Goodbye
All
GasGauge
WiFi
Shutdown
Restart
MobileGestalt
Sleep
NAND
IORegistry
Obliterate
"""

"""
BasebandKeyHashInformation
BasebandFirmwareManifestData
"""

MobileGestaltKeys = """DieId
SerialNumber
UniqueChipID
WifiAddress
CPUArchitecture
BluetoothAddress
EthernetMacAddress
FirmwareVersion
MLBSerialNumber
ModelNumber
RegionInfo
RegionCode
DeviceClass
ProductType
DeviceName
UserAssignedDeviceName
HWModelStr
SigningFuse
SoftwareBehavior
SupportedKeyboards
BuildVersion
ProductVersion
ReleaseType
InternalBuild
CarrierInstallCapability
IsUIBuild
InternationalMobileEquipmentIdentity
MobileEquipmentIdentifier
DeviceColor
HasBaseband
SupportedDeviceFamilies
SoftwareBundleVersion
SDIOManufacturerTuple
SDIOProductInfo
UniqueDeviceID
InverseDeviceID
ChipID
PartitionType
ProximitySensorCalibration
CompassCalibration
WirelessBoardSnum
BasebandBoardSnum
HardwarePlatform
RequiredBatteryLevelForSoftwareUpdate
IsThereEnoughBatteryLevelForSoftwareUpdate
BasebandRegionSKU
encrypted-data-partition
SysCfg
DiagData
SIMTrayStatus
CarrierBundleInfoArray
AllDeviceCapabilities
wi-fi
SBAllowSensitiveUI
green-tea
not-green-tea
AllowYouTube
AllowYouTubePlugin
SBCanForceDebuggingInfo
AppleInternalInstallCapability
HasAllFeaturesCapability
ScreenDimensions
IsSimulator
BasebandSerialNumber
BasebandChipId
BasebandCertId
BasebandSkeyId
BasebandFirmwareVersion
cellular-data
contains-cellular-radio
RegionalBehaviorGoogleMail
RegionalBehaviorVolumeLimit
RegionalBehaviorShutterClick
RegionalBehaviorNTSC
RegionalBehaviorNoWiFi
RegionalBehaviorChinaBrick
RegionalBehaviorNoVOIP
RegionalBehaviorAll
ApNonce"""


class DIAGClient(object):

    def __init__(self, lockdown=None, serviceName="com.apple.mobile.diagnostics_relay"):
        if lockdown:
            self.lockdown = lockdown
        else:
            self.lockdown = LockdownClient()

        self.service = self.lockdown.startService(serviceName)
        self.packet_num = 0

    def stop_session(self):
        print("Disconecting...")
        self.service.close()

    def query_mobilegestalt(self, MobileGestalt=MobileGestaltKeys.split("\n")):
        self.service.sendPlist({"Request": "MobileGestalt",
                                "MobileGestaltKeys": MobileGestalt})

        res = self.service.recvPlist()
        d = res.get("Diagnostics")
        if d:
            return d.get("MobileGestalt")
        return None

    def action(self, action="Shutdown", flags=None):
        self.service.sendPlist({"Request": action})
        res = self.service.recvPlist()
        return res.get("Diagnostics")

    def restart(self):
        return self.action("Restart")

    def shutdown(self):
        return self.action("Shutdown")

    def diagnostics(self, diagType="All"):
        self.service.sendPlist({"Request": diagType})
        res = self.service.recvPlist()
        if res:
            return res.get("Diagnostics")
        return None

    def ioregistry_entry(self, name=None, ioclass=None):
        d = {}
        if name:
            d["EntryName"] = name

        if ioclass:
            d["EntryClass"] = ioclass

        d["Request"] = "IORegistry"

        self.service.sendPlist(d)
        res = self.service.recvPlist()
        pprint(res)
        if res:
            return res.get("Diagnostics")
        return None

    def ioregistry_plane(self, plane=""):
        d = {}
        if plane:
            d["CurrentPlane"] = plane

        else:
            d["CurrentPlane"] = ""
        d["Request"] = "IORegistry"

        self.service.sendPlist(d)
        res = self.service.recvPlist()
        dd = res.get("Diagnostics")
        if dd:
            return dd.get("IORegistry")
        return None


if __name__ == "__main__":

    parser = OptionParser(usage="%prog")
    parser.add_option("-c", "--cmd", dest="cmd", default=False,
                      help="Launch diagnostic command", type="string")
    parser.add_option("-m", "--mobilegestalt", dest="mobilegestalt_key", default=False,
                      help="Request mobilegestalt key", type="string")
    parser.add_option("-i", "--ioclass", dest="ioclass", default=False,
                      help="Request ioclass", type="string")
    parser.add_option("-n", "--ioname", dest="ioname", default=False,
                      help="Request ionqme", type="string")

    (options, args) = parser.parse_args()

    diag = DIAGClient()
    if not options.cmd:
        res = diag.diagnostics()

    elif options.cmd == "IORegistry":
        res = diag.ioregistry_plane()

    elif options.cmd == "MobileGestalt":

        if not options.mobilegestalt_key or options.mobilegestalt_key not in MobileGestaltKeys.split("\n"):
            res = diag.query_mobilegestalt()

        else:
            res = diag.query_mobilegestalt([options.mobilegestalt_key])

    else:
        res = diag.action(options.cmd)

    if res:
        for k in list(res.keys()):
            print(" %s \t: %s" % (k, res[k]))



#!/usr/bin/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#


from construct.core import Struct
from construct.lib.container import Container
from construct.macros import String, ULInt64
from pymobiledevice.lockdown import LockdownClient
import struct
from cmd import Cmd
import os
from pymobiledevice.util import hexdump, parsePlist
from pprint import pprint
import plistlib
import posixpath

MODEMASK = 0o0000777

AFC_OP_STATUS = 0x00000001
AFC_OP_DATA = 0x00000002  # Data */
AFC_OP_READ_DIR = 0x00000003  # ReadDir */
AFC_OP_READ_FILE = 0x00000004  # ReadFile */
AFC_OP_WRITE_FILE = 0x00000005  # WriteFile */
AFC_OP_WRITE_PART = 0x00000006  # WritePart */
AFC_OP_TRUNCATE = 0x00000007  # TruncateFile */
AFC_OP_REMOVE_PATH = 0x00000008  # RemovePath */
AFC_OP_MAKE_DIR = 0x00000009  # MakeDir */
AFC_OP_GET_FILE_INFO = 0x0000000a  # GetFileInfo */
AFC_OP_GET_DEVINFO = 0x0000000b  # GetDeviceInfo */
AFC_OP_WRITE_FILE_ATOM = 0x0000000c  # WriteFileAtomic (tmp file+rename) */
AFC_OP_FILE_OPEN = 0x0000000d  # FileRefOpen */
AFC_OP_FILE_OPEN_RES = 0x0000000e  # FileRefOpenResult */
AFC_OP_READ = 0x0000000f  # FileRefRead */
AFC_OP_WRITE = 0x00000010  # FileRefWrite */
AFC_OP_FILE_SEEK = 0x00000011  # FileRefSeek */
AFC_OP_FILE_TELL = 0x00000012  # FileRefTell */
AFC_OP_FILE_TELL_RES = 0x00000013  # FileRefTellResult */
AFC_OP_FILE_CLOSE = 0x00000014  # FileRefClose */
AFC_OP_FILE_SET_SIZE = 0x00000015  # FileRefSetFileSize (ftruncate) */
AFC_OP_GET_CON_INFO = 0x00000016  # GetConnectionInfo */
AFC_OP_SET_CON_OPTIONS = 0x00000017  # SetConnectionOptions */
AFC_OP_RENAME_PATH = 0x00000018  # RenamePath */
AFC_OP_SET_FS_BS = 0x00000019  # SetFSBlockSize (0x800000) */
AFC_OP_SET_SOCKET_BS = 0x0000001A  # SetSocketBlockSize (0x800000) */
AFC_OP_FILE_LOCK = 0x0000001B  # FileRefLock */
AFC_OP_MAKE_LINK = 0x0000001C  # MakeLink */
AFC_OP_SET_FILE_TIME = 0x0000001E  # set st_mtime */

AFC_E_SUCCESS = 0
AFC_E_UNKNOWN_ERROR = 1
AFC_E_OP_HEADER_INVALID = 2
AFC_E_NO_RESOURCES = 3
AFC_E_READ_ERROR = 4
AFC_E_WRITE_ERROR = 5
AFC_E_UNKNOWN_PACKET_TYPE = 6
AFC_E_INVALID_ARG = 7
AFC_E_OBJECT_NOT_FOUND = 8
AFC_E_OBJECT_IS_DIR = 9
AFC_E_PERM_DENIED = 10
AFC_E_SERVICE_NOT_CONNECTED = 11
AFC_E_OP_TIMEOUT = 12
AFC_E_TOO_MUCH_DATA = 13
AFC_E_END_OF_DATA = 14
AFC_E_OP_NOT_SUPPORTED = 15
AFC_E_OBJECT_EXISTS = 16
AFC_E_OBJECT_BUSY = 17
AFC_E_NO_SPACE_LEFT = 18
AFC_E_OP_WOULD_BLOCK = 19
AFC_E_IO_ERROR = 20
AFC_E_OP_INTERRUPTED = 21
AFC_E_OP_IN_PROGRESS = 22
AFC_E_INTERNAL_ERROR = 23

AFC_E_MUX_ERROR = 30
AFC_E_NO_MEM = 31
AFC_E_NOT_ENOUGH_DATA = 32
AFC_E_DIR_NOT_EMPTY = 33

AFC_FOPEN_RDONLY = 0x00000001  # /**< r   O_RDONLY */
AFC_FOPEN_RW = 0x00000002  # /**< r+  O_RDWR   | O_CREAT */
AFC_FOPEN_WRONLY = 0x00000003  # /**< w   O_WRONLY | O_CREAT  | O_TRUNC */
AFC_FOPEN_WR = 0x00000004  # /**< w+  O_RDWR   | O_CREAT  | O_TRUNC */
AFC_FOPEN_APPEND = 0x00000005  # /**< a   O_WRONLY | O_APPEND | O_CREAT */
AFC_FOPEN_RDAPPEND = 0x00000006  # /**< a+  O_RDWR   | O_APPEND | O_CREAT */

AFC_HARDLINK = 1
AFC_SYMLINK = 2

AFC_LOCK_SH = 1 | 4  # /**< shared lock */
AFC_LOCK_EX = 2 | 4  # /**< exclusive lock */
AFC_LOCK_UN = 8 | 4  # /**< unlock */


AFCMAGIC = "CFA6LPAA"
AFCPacket = Struct("AFCPacket",
                   String("magic", 8,),
                   ULInt64("entire_length"),
                   ULInt64("this_length"),
                   ULInt64("packet_num"),
                   ULInt64("operation")
                   )


class AFCClient(object):

    def __init__(self, lockdown=None, serviceName="com.apple.afc", service=None):
        self.serviceName = serviceName
        self.lockdown = lockdown if lockdown else LockdownClient()
        self.service = service if service else self.lockdown.startService(
            self.serviceName)
        self.packet_num = 0

    def stop_session(self):
        print("Disconecting...")
        self.service.close()

    def dispatch_packet(self, operation, data, this_length=0):
        afcpack = Container(magic=AFCMAGIC,
                            entire_length=40 + len(data),
                            this_length=40 + len(data),
                            packet_num=self.packet_num,
                            operation=operation)
        if this_length:
            afcpack.this_length = this_length
        header = AFCPacket.build(afcpack)
        self.packet_num += 1
        if isinstance(data, str):
            data = data.encode()
        self.service.send(header + data)

    def receive_data(self):
        res = self.service.recv(40)
        status = AFC_E_SUCCESS
        data = ""
        if res:
            res = AFCPacket.parse(res)
            assert res["entire_length"] >= 40
            length = res["entire_length"] - 40
            data = self.service.recv_exact(length)
            if res.operation == AFC_OP_STATUS:
                if length != 8:
                    print("Status length != 8")
                status = struct.unpack("<Q", data[:8])[0]
            elif res.operation != AFC_OP_DATA:
                pass  # print "error ?", res
        return status, data

    def do_operation(self, opcode, data=""):
        try:
            self.dispatch_packet(opcode, data)
            return self.receive_data()
        except ImportError:
            self.lockdown = LockdownClient()
            self.service = self.lockdown.startService(self.serviceName)
            return self.do_operation(opcode, data)

    def list_to_dict(self, d):
        t = d.split("\x00")
        t = t[:-1]

        assert len(t) % 2 == 0
        res = {}
        for i in range(len(t) / 2):
            res[t[i * 2]] = t[i * 2 + 1]
        return res

    def get_device_infos(self):
        status, infos = self.do_operation(AFC_OP_GET_DEVINFO)
        if status == AFC_E_SUCCESS:
            return self.list_to_dict(infos)

    def read_directory(self, dirname):
        status, data = self.do_operation(AFC_OP_READ_DIR, dirname)
        if status == AFC_E_SUCCESS:
            return [x for x in data.split("\x00") if x != ""]
        return []

    def make_directory(self, dirname):
        status, data = self.do_operation(AFC_OP_MAKE_DIR, dirname)
        return status

    def remove_directory(self, dirname):
        info = self.get_file_info(dirname)
        if not info or info.get("st_ifmt") != "S_IFDIR":
            print("remove_directory: %s not S_IFDIR" % dirname)
            return

        for d in self.read_directory(dirname):
            if d == "." or d == ".." or d == "":
                continue

            info = self.get_file_info(dirname + "/" + d)
            if info.get("st_ifmt") == "S_IFDIR":
                self.remove_directory(dirname + "/" + d)
            else:
                print(dirname + "/" + d)
                self.file_remove(dirname + "/" + d)
        assert len(self.read_directory(dirname)) == 2  # .. et .
        return self.file_remove(dirname)

    def get_file_info(self, filename):
        status, data = self.do_operation(AFC_OP_GET_FILE_INFO, filename)
        if status == AFC_E_SUCCESS:
            return self.list_to_dict(data)

    def make_link(self, target, linkname, type=AFC_SYMLINK):
        status, data = self.do_operation(AFC_OP_MAKE_LINK, struct.pack(
            "<Q", type) + target + "\x00" + linkname + "\x00")
        print("make_link", status)
        return status

    def file_open(self, filename, mode=AFC_FOPEN_RDONLY):
        status, data = self.do_operation(
            AFC_OP_FILE_OPEN, struct.pack("<Q", mode).decode() + filename + "\x00")
        return struct.unpack("<Q", data)[0] if data else None

    def file_close(self, handle):
        status, data = self.do_operation(
            AFC_OP_FILE_CLOSE, struct.pack("<Q", handle))
        return status

    def file_remove(self, filename):
        status, data = self.do_operation(AFC_OP_REMOVE_PATH, filename + "\x00")
        return status

    def file_rename(self, old, new):
        status, data = self.do_operation(
            AFC_OP_RENAME_PATH, old + "\x00" + new + "\x00")
        return status

    def file_read(self, handle, sz):
        MAXIMUM_READ_SIZE = 1 << 16
        data = ""
        while sz > 0:
            if sz > MAXIMUM_READ_SIZE:
                toRead = MAXIMUM_READ_SIZE
            else:
                toRead = sz
            try:
                self.dispatch_packet(
                    AFC_OP_READ, struct.pack("<QQ", handle, toRead))
                s, d = self.receive_data()
            except:
                self.lockdown = LockdownClient()
                self.service = self.lockdown.startService("com.apple.afc")
                return self.file_read(handle, sz)

            if s != AFC_E_SUCCESS:
                break
            sz -= toRead
            data += d
        return data

    def file_write(self, handle, data):
        MAXIMUM_WRITE_SIZE = 1 << 15
        hh = struct.pack("<Q", handle)
        segments = int(len(data) / MAXIMUM_WRITE_SIZE)
        try:
            for i in range(segments):
                self.dispatch_packet(AFC_OP_WRITE,
                                     hh +
                                     data[
                                         i * MAXIMUM_WRITE_SIZE:(i + 1) * MAXIMUM_WRITE_SIZE],
                                     this_length=48)
                s, d = self.receive_data()
                if s != AFC_E_SUCCESS:
                    print("file_write error %d" % s)
                    break
            if len(data) % MAXIMUM_WRITE_SIZE:
                self.dispatch_packet(AFC_OP_WRITE,
                                     hh + data[segments * MAXIMUM_WRITE_SIZE:],
                                     this_length=48)
                s, d = self.receive_data()
        except ImportError:
            self.lockdown = LockdownClient()
            self.service = self.lockdown.startService(serviceName)
            self.file_write(handle, data)
        return s

    def get_file_contents(self, filename):
        info = self.get_file_info(filename)
        if info:
            if info['st_ifmt'] == 'S_IFLNK':
                filename = info['LinkTarget']

            if info['st_ifmt'] == 'S_IFDIR':
                print("%s is directory..." % filename)
                return

            print("Reading %s" % filename)
            h = self.file_open(filename)
            if not h:
                return
            d = self.file_read(h, int(info["st_size"]))
            self.file_close(h)
            return d

    def set_file_contents(self, filename, data):
        h = self.file_open(filename, AFC_FOPEN_WR)
        if not h:
            return
        d = self.file_write(h, data)
        self.file_close(h)

    def dir_walk(self, dirname):
        dirs = []
        files = []
        for fd in self.read_directory(dirname):
            if fd in ('.', '..', ''):
                continue
            infos = self.get_file_info(posixpath.join(dirname, fd))
            if infos and infos.get('st_ifmt') == 'S_IFDIR':
                dirs.append(fd)
            else:
                files.append(fd)

        yield dirname, dirs, files

        if dirs:
            for d in dirs:
                for walk_result in self.dir_walk(posixpath.join(dirname, d)):
                    yield walk_result


class AFCShell(Cmd):

    def __init__(self, afcname='com.apple.afc', completekey='tab', stdin=None, stdout=None, client=None):

        Cmd.__init__(self, completekey=completekey, stdin=stdin, stdout=stdout)
        self.lockdown = LockdownClient()
        self.afc = client if client else AFCClient(
            self.lockdown, serviceName=afcname)

        self.curdir = '/'
        self.prompt = 'AFC$ ' + self.curdir + ' '
        self.complete_cat = self._complete
        self.complete_ls = self._complete

    def do_exit(self, p):
        return True

    def do_quit(self, p):
        return True

    def do_pwd(self, p):
        print(self.curdir)

    def do_link(self, p):
        z = p.split()
        self.afc.make_link(AFC_SYMLINK, z[0], z[1])

    def do_cd(self, p):
        if not p.startswith("/"):
            new = self.curdir + "/" + p
        else:
            new = p

        new = os.path.normpath(new).replace("\\", "/").replace("//", "/")
        if self.afc.read_directory(new):
            self.curdir = new
            self.prompt = "AFC$ %s " % new
        else:
            print("%s does not exist" % new)

    def _complete(self, text, line, begidx, endidx):
        filename = text.split("/")[-1]
        dirname = "/".join(text.split("/")[:-1])
        return [dirname + "/" + x for x in self.afc.read_directory(self.curdir + "/" + dirname) if x.startswith(filename)]

    def do_ls(self, p):
        d = self.afc.read_directory(self.curdir + "/" + p)
        if d:
            for dd in d:
                print(dd)

    def do_cat(self, p):
        data = self.afc.get_file_contents(self.curdir + "/" + p)
        if data and p.endswith(".plist"):
            pprint(parsePlist(data))
        else:
            print(data)

    def do_rm(self, p):
        f = self.afc.get_file_info(self.curdir + "/" + p)
        if f['st_ifmt'] == 'S_IFDIR':
            d = self.afc.remove_directory(self.curdir + "/" + p)
        else:
            d = self.afc.file_remove(self.curdir + "/" + p)

    def do_pull(self, user_args):
        args = user_args.split()
        if len(args) != 2:
            out = "."
            path = user_args
        else:
            out = args[1]
            path = args[0]

        f = self.afc.get_file_info(path)
        if not f:
            print("Source file does not exist..")
            return

        out_path = out + "/" + path
        if f['st_ifmt'] == 'S_IFDIR':

            if not os.path.isdir(out_path):
                os.makedirs(out_path, MODEMASK)

            for d in self.afc.read_directory(path):
                if d == "." or d == ".." or d == "":
                    continue
                self.do_pull(path + "/" + d + " " + out)
        else:
            data = self.afc.get_file_contents(path)
            if data:
                if data and path.endswith(".plist"):
                    z = parsePlist(data)
                    plistlib.writePlist(z, out_path)
                else:
                    with open(out_path, 'wb+') as f:
                        f.write(data)

    def do_push(self, p):
        fromTo = p.split()
        if len(fromTo) != 2:
            return
        print("from %s to %s" % (fromTo[0], fromTo[1]))
        if os.path.isdir(fromTo[0]):
            self.afc.make_directory(os.path.join(fromTo[1]))
            for x in os.listdir(fromTo[0]):
                if x.startswith("."):
                    continue
                path = os.path.join(fromTo[0], x)
                self.do_push(path + " " + fromTo[1] + "/" + path)
        else:
            if not fromTo[0].startswith("."):
                data = open(fromTo[0], "rb").read()
                self.afc.set_file_contents(self.curdir + "/" + fromTo[1], data)

    def do_head(self, p):
        print(self.afc.get_file_contents(self.curdir + "/" + p)[:32])

    def do_hexdump(self, p):
        t = p.split(" ")
        l = 0
        if len(t) < 1:
            return
        if len(t) == 2:
            l = int(t[1])
        z = self.afc.get_file_contents(self.curdir + "/" + t[0])
        if not z:
            return
        if l:
            z = z[:l]
        hexdump(z)

    def do_mkdir(self, p):
        print(self.afc.make_directory(p))

    def do_rmdir(self, p):
        return self.afc.remove_directory(p)

    def do_infos(self, p):
        print(self.afc.get_device_infos())

    def do_rmdir(self, p):
        return self.afc.remove_directory(p)

    def do_mv(self, p):
        t = p.split()
        return self.afc.rename_path(t[0], t[1])


class AFC2Client(AFCClient):

    def __init__(self, lockdown=None):
        super(AFC, self).__init__(lockdown, serviceName="com.apple.afc2")


class AFCCrashLog(AFCClient):

    def __init__(self, lockdown=None):
        super(AFCCrashLog, self).__init__(
            lockdown, serviceName="com.apple.crashreportcopymobile")


if __name__ == "__main__":
    AFCShell().cmdloop("Hello iPhone!")



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
File name: apis.py
Author: dhilipsiva <dhilipsiva@gmail.com>
Date created: 2016-06-19
"""

from os import path
# from pprint import pprint

from pymobiledevice.afc import AFCClient
from pymobiledevice.lockdown import LockdownClient


class AppManager(object):
    """
    The App Manager
    """

    def __init__(self, udid):
        super(AppManager, self).__init__()
        self.udid = udid
        self.lockdown = LockdownClient(udid)
        self.service = self.lockdown.startService(
            "com.apple.mobile.installation_proxy")

    def run_command(self, cmd):
        self.service.sendPlist(cmd)
        z = self.service.recvPlist()
        while 'PercentComplete' in z:
            if not z:
                break
            if z.get("Status") == "Complete":
                return z.get("Status")
            z = self.service.recvPlist()
        return z

    def install_ipa(self, ipa_path):
        """
        docstring for install_ipa
        """
        afc = AFCClient(lockdown=self.lockdown)
        afc.set_file_contents(
            path.basename(ipa_path), open(ipa_path, "rb").read())
        cmd = {"Command": "Install", "PackagePath": path.basename(ipa_path)}
        return self.run_command(cmd)

    def uninstall_ipa(self, bundle_id):
        cmd = {"Command": "Uninstall", "ApplicationIdentifier": bundle_id}
        return self.run_command(cmd)

    def list_ipas(self):
        cmd = {"Command": "Lookup"}
        result = self.run_command(cmd)
        apps_details = result.get("LookupResult")
        apps = []
        for app in apps_details:
            if apps_details[app]['ApplicationType'] == 'User':
                apps.append(app)
        return apps



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
File name: setup.py
Version: 0.1
Author: dhilipsiva <dhilipsiva@gmail.com>
Date created: 2015-11-24
"""

from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))
f = path.join(here, 'README.md')

setup(
    name='pymobiledevice',
    version='0.0.4',
    description="Interface with iOS devices",
    url='https://github.com/appknox/pymobiledevice',
    author='dhilipsiva',
    author_email='dhilipsiva@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='pymobiledevice ios iphone ipad ipod',
    packages=find_packages(),
    py_modules=['pymobiledevice'],
    entry_points='',
    install_requires=[
        'ak-vendor',
        'ak-construct',
        'ak-m2crypto',
    ],
    extras_require={
        'dev': [''],
        'test': [''],
    },
)



#!/usr/bin/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

from pprint import pprint
import sys
from struct import unpack, pack
import os

# Values for mode, OR'd together:

ISDIR = 0o040000  # Directory
ISFIFO = 0o010000  # FIFO
ISREG = 0o0100000  # Regular file
ISBLK = 0o060000  # Block special file
ISCHR = 0o020000  # Character special file
ISCTG = 0o0110000  # Reserved for contiguous files
ISLNK = 0o0120000  # Reserved for symbolic links
ISOCK = 0o0140000  # Reserved for sockets
IFMT = 0o0170000  # type of file

MODEMASK = 0o0000777
TRAILER = "TRAILER!!!"

NEW_MAGIC = 0o070701  # New ASCII magic
CRC_MAGIC = 0o070702  # New CRC magic
OLD_MAGIC = 0o070707  # Old ASCII magic


def version():
    return '0.1'


class CpioArchive(object):

    def __init__(self, cpiofile=None, fileobj=None, mode="rb"):
        if fileobj:
            self.ifile = fileobj
        else:
            self.ifile = open(cpiofile, mode)

    def is_cpiofile(self, cpiofile=None, fileobj=None):
        if fileobj:
            magic = int(fileobj.read(6), 8)
        else:
            magic = int(open(cpiofile, 'r').read(6), 8)

        if magic in [NEW_MAGIC, CRC_MAGIC, OLD_MAGIC]:
            return True

    def read_old_ascii_cpio_record(self):
        f = {}
        try:
            f["dev"] = int(self.ifile.read(6), 8)  # device where file resides
            f["ino"] = int(self.ifile.read(6), 8)  # I-number of file
            f["mode"] = int(self.ifile.read(6), 8)  # Ifile mode
            f["uid"] = int(self.ifile.read(6), 8)  # owner user ID
            f["gid"] = int(self.ifile.read(6), 8)  # owner group ID
            f["nlink"] = int(self.ifile.read(6), 8)  # number of links to file
            # device major/minor for special file
            f["rdev"] = int(self.ifile.read(6), 8)
            f["mtime"] = int(self.ifile.read(11), 8)  # modify time of file
            f["namesize"] = int(self.ifile.read(6), 8)  # length of file name
            # length of file to follow
            f["filesize"] = int(self.ifile.read(11), 8)
            f["name"] = self.ifile.read(f.get("namesize"))[
                :-1]  # Removing \x00
            f["data"] = self.ifile.read(f.get("filesize"))
        except:
            print('ERROR: cpio record trunked (incomplete archive)')
            return None
        return f

    def extract_files(self, files=None, outpath="."):
        print("Extracting files from CPIO archive")
        while 1:
            try:
                hdr = int(self.ifile.read(6), 8)
            except:
                print('ERROR: cpio record trunked (incomplete archive)')
                break

            if hdr != OLD_MAGIC:
                raise NotImplementedError  # FIXME Should implement new & Binary CPIO record

            f = self.read_old_ascii_cpio_record()
            if f and f.get("name") == TRAILER:
                break

            if files:
                if not f.get("name") in files:
                    print("Skipped %s" % f.get("name"))
                    continue

            fullOutPath = os.path.join(outpath, f.get("name").strip("../"))
            print("x %s" % fullOutPath)

            if (f.get("mode") & IFMT == ISFIFO):  # FIFO
                if not os.path.isdir(os.path.dirname(fullOutPath)):
                    os.makedirs(os.path.dirname(fullOutPath), 0o0755)
                os.mkfifo(fullOutPath, f.get("mode") & MODEMASK)
                os.chmod(fullOutPath, f.get("mode") & MODEMASK)

            if (f.get("mode") & IFMT == ISDIR):  # Directory
                if not os.path.isdir(fullOutPath):
                    os.makedirs(fullOutPath, f.get("mode") & MODEMASK)

            if (f.get("mode") & IFMT == ISBLK):  # Block special file
                raise NotImplementedError

            if (f.get("mode") & IFMT == ISCHR):  # Character special file
                raise NotImplementedError

            if (f.get("mode") & IFMT == ISLNK):  # Reserved for symbolic links
                raise NotImplementedError

            if (f.get("mode") & IFMT == ISOCK):  # Reserved for sockets
                raise NotImplementedError

            # Contiguous or Regular file
            if (f.get("mode") & IFMT == ISCTG) or (f.get("mode") & IFMT == ISREG):
                if not os.path.isdir(os.path.dirname(fullOutPath)):
                    os.makedirs(os.path.dirname(fullOutPath), 0o0755)
                fd = open(fullOutPath, "wb")
                fd.write(f.get("data"))

            os.chmod(fullOutPath, f.get("mode") & MODEMASK)


if __name__ == "__main__":
    a = CpioArchive(sys.argv[1], mode="rb",)
    a.extract_files()



#!/usr/bin/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#


import os
import hashlib
import datetime
from optparse import OptionParser
from pprint import pprint
from pymobiledevice.util import write_file, hexdump
from biplist import writePlist, readPlist, Data
from struct import unpack, pack
from time import mktime, gmtime, sleep, time

from pymobiledevice.lockdown import LockdownClient
from pymobiledevice.mobilebackup import MobileBackup

CODE_SUCCESS = 0x00
CODE_ERROR_LOCAL = 0x06
CODE_ERROR_REMOTE = 0x0b
CODE_FILE_DATA = 0x0c

ERROR_ENOENT = -6
ERROR_EEXIST = -7


class DeviceVersionNotSupported(Exception):

    def __str__(self):
        return "Device version not supported, please use mobilebackup"


class MobileBackup2(MobileBackup):

    service = None

    def __init__(self, lockdown=None, backupPath=None, password=""):
        if lockdown:
            self.lockdown = lockdown
        else:
            self.lockdown = LockdownClient()

        ProductVersion = self.lockdown.getValue("", "ProductVersion")
        if ProductVersion[0] < "5":
            raise DeviceVersionNotSupported

        self.udid = lockdown.getValue("", "UniqueDeviceID")
        self.willEncrypt = lockdown.getValue(
            "com.apple.mobile.backup", "WillEncrypt")
        self.escrowBag = lockdown.getValue('', 'EscrowBag')

        self.service = self.lockdown.startServiceWithEscrowBag(
            "com.apple.mobilebackup2", self.escrowBag)
        if not self.service:
            raise Exception(
                "MobileBackup2 init error : Could not start com.apple.mobilebackup2")

        if backupPath:
            self.backupPath = backupPath
        else:
            self.backupPath = "backups"
        if not os.path.isdir(self.backupPath):
            os.makedirs(self.backupPath, 0o0755)

        print("Starting new com.apple.mobilebackup2 service with working dir: %s" %
              self.backupPath)

        self.password = password
        DLMessageVersionExchange = self.service.recvPlist()
        version_major = DLMessageVersionExchange[1]
        self.service.sendPlist(
            ["DLMessageVersionExchange", "DLVersionsOk", version_major])
        DLMessageDeviceReady = self.service.recvPlist()
        if DLMessageDeviceReady and DLMessageDeviceReady[0] == "DLMessageDeviceReady":
            self.version_exchange()
        else:
            raise Exception("MobileBackup2 init error %s" %
                            DLMessageDeviceReady)

    def __del__(self):
        if self.service:
            self.service.sendPlist(
                ["DLMessageDisconnect", "___EmptyParameterString___"])

    def internal_mobilebackup2_send_message(self, name, data):
        data["MessageName"] = name
        self.device_link_service_send_process_message(data)

    def internal_mobilebackup2_receive_message(self, name=None):
        res = self.device_link_service_receive_process_message()
        if res:
            if name and res["MessageName"] != name:
                print("MessageName does not match %s %s" % (name, str(res)))
            return res

    def version_exchange(self):
        self.internal_mobilebackup2_send_message(
            "Hello", {"SupportedProtocolVersions": [2.0, 2.1]})
        return self.internal_mobilebackup2_receive_message("Response")

    def mobilebackup2_send_request(self, request, target, source, options={}):
        d = {"TargetIdentifier": target,
             "SourceIdentifier": source,
             "Options": options}
        self.internal_mobilebackup2_send_message(request, d)

    def mobilebackup2_receive_message(self):
        return self.service.recvPlist()

    def mobilebackup2_send_status_response(self, status_code, status1="___EmptyParameterString___", status2={}):
        a = ["DLMessageStatusResponse", status_code, status1, status2]
        self.service.sendPlist(a)

    def mb2_handle_free_disk_space(self, msg):
        s = os.statvfs(self.backupPath)
        freeSpace = s.f_bsize * s.f_bavail
        a = ["DLMessageStatusResponse", 0, freeSpace]
        self.service.sendPlist(a)

    def mb2_multi_status_add_file_error(self, errplist, path, error_code, error_message):
        errplist[path] = {"DLFileErrorCode": error_code,
                          "DLFileErrorString": error_message}

    def mb2_handle_copy_item(self, msg):
        src = self.check_filename(msg[1])
        dst = self.check_filename(msg[2])
        if os.path.isfile(src):
            data = self.read_file(src)
            self.write_file(dst, data)
        else:
            os.makedirs(dst)
        self.mobilebackup2_send_status_response(0)

    def mb2_handle_send_file(self, filename, errplist):
        self.service.send_raw(filename)
        if not filename.startswith(self.udid):
            filename = self.udid + "/" + filename

        data = self.read_file(self.check_filename(filename))
        if data != None:
            print("Sending %s to device" % filename)
            self.service.send_raw(chr(CODE_FILE_DATA) + data)
            self.service.send_raw(chr(CODE_SUCCESS))
        else:
            print("File %s requested from device not found" % filename)
            self.service.send_raw(chr(CODE_ERROR_LOCAL))
            self.mb2_multi_status_add_file_error(
                errplist, filename, ERROR_ENOENT, "Could not find the droid you were looking for ;)")

    def mb2_handle_send_files(self, msg):
        errplist = {}
        for f in msg[1]:
            self.mb2_handle_send_file(f, errplist)
        self.service.send("\x00\x00\x00\x00")
        if len(errplist):
            self.mobilebackup2_send_status_response(
                -13, "Multi status", errplist)
        else:
            self.mobilebackup2_send_status_response(0)

    def mb2_handle_list_directory(self, msg):
        path = msg[1]
        dirlist = {}
        self.mobilebackup2_send_status_response(0, status2=dirlist)

    def mb2_handle_make_directory(self, msg):
        dirname = self.check_filename(msg[1])
        print("Creating directory %s" % dirname)
        if not os.path.isdir(dirname):
            os.makedirs(dirname)
        self.mobilebackup2_send_status_response(0, "")

    def mb2_handle_receive_files(self, msg):
        done = 0
        while not done:
            device_filename = self.service.recv_raw()
            if device_filename == "":
                break
            backup_filename = self.service.recv_raw()
            filedata = ""
            while True:
                stuff = self.service.recv_raw()
                if ord(stuff[0]) == CODE_FILE_DATA:
                    filedata += stuff[1:]
                elif ord(stuff[0]) == CODE_SUCCESS:
                    self.write_file(self.check_filename(
                        backup_filename), filedata)
                    break
                else:
                    print("Unknown code", ord(stuff[0]))
                    break
        self.mobilebackup2_send_status_response(0)

    def mb2_handle_move_files(self, msg):
        for k, v in list(msg[1].items()):
            print("Renaming %s to %s" %
                  (self.check_filename(k), self.check_filename(v)))
            os.rename(self.check_filename(k), self.check_filename(v))
        self.mobilebackup2_send_status_response(0)

    def mb2_handle_remove_files(self, msg):
        for filename in msg[1]:
            print("Removing ", self.check_filename(filename))
            try:
                filename = self.check_filename(filename)
                if os.path.isfile(filename):
                    os.unlink(filename)
            except Exception as e:
                print(e)
        self.mobilebackup2_send_status_response(0)

    def work_loop(self):
        while True:
            msg = self.mobilebackup2_receive_message()
            if not msg:
                break

            assert(msg[0] in ["DLMessageDownloadFiles",
                              "DLContentsOfDirectory",
                              "DLMessageCreateDirectory",
                              "DLMessageUploadFiles",
                              "DLMessageMoveFiles", "DLMessageMoveItems",
                              "DLMessageRemoveFiles", "DLMessageRemoveItems",
                              "DLMessageCopyItem",
                              "DLMessageProcessMessage",
                              "DLMessageGetFreeDiskSpace",
                              "DLMessageDisconnect"])

            if msg[0] == "DLMessageDownloadFiles":
                self.mb2_handle_send_files(msg)
            elif msg[0] == "DLContentsOfDirectory":
                self.mb2_handle_list_directory(msg)
            elif msg[0] == "DLMessageCreateDirectory":
                self.mb2_handle_make_directory(msg)
            elif msg[0] == "DLMessageUploadFiles":
                self.mb2_handle_receive_files(msg)
            elif msg[0] in ["DLMessageMoveFiles", "DLMessageMoveItems"]:
                self.mb2_handle_move_files(msg)
            elif msg[0] in ["DLMessageRemoveFiles", "DLMessageRemoveItems"]:
                self.mb2_handle_remove_files(msg)
            elif msg[0] == "DLMessageCopyItem":
                self.mb2_handle_copy_item(msg)
            elif msg[0] == "DLMessageProcessMessage":
                errcode = msg[1].get("ErrorCode")
                if errcode == 0:
                    m = msg[1].get("MessageName")
                    if m != "Response":
                        print(m)
                if errcode == 1:
                    print(msg[1].get("ErrorDescription"))
                    print("Please unlock your device and retry...")
                break
            elif msg[0] == "DLMessageGetFreeDiskSpace":
                self.mb2_handle_free_disk_space(msg)
            elif msg[0] == "DLMessageDisconnect":
                break

    def create_status_plist(self, fullBackup=True):
        # Creating Status file for backup
        statusDict = {'UUID': '82D108D4-521C-48A5-9C42-79C5E654B98F',  # FixMe We Should USE an UUID generator uuid.uuid3(uuid.NAMESPACE_DNS, hostname)
                      'BackupState': 'new',
                      'IsFullBackup': fullBackup,
                      'Version': '2.4',
                      'Date': datetime.datetime.fromtimestamp(mktime(gmtime())),
                      'SnapshotState': 'finished'
                      }
        writePlist(statusDict, self.check_filename("Status.plist"))

    def backup(self, fullBackup=True):
        print("Starting%sbackup..." %
              (" Encrypted " if self.willEncrypt else ""))
        options = {}
        if not os.path.isdir(os.path.join(self.backupPath, self.udid)):
            os.makedirs(os.path.join(self.backupPath, self.udid))

        self.create_info_plist()

        if fullBackup == True:
            options["ForceFullBackup"] = True
        self.mobilebackup2_send_request("Backup", self.udid, options)
        self.work_loop()

    def restore(self, options={"RestoreSystemFiles": True,
                               "RestoreShouldReboot": False,
                               "RestoreDontCopyBackup": True,
                               "RestorePreserveSettings": True},
                password=None):

        print("Starting restoration...")
        m = os.path.join(self.backupPath, self.udid, "Manifest.plist")
        manifest = readPlist(m)
        if manifest.get("IsEncrypted"):
            print("Backup is encrypted, enter password : ")
            if password:
                self.password = password
            else:
                self.password = input()
            options["Password"] = self.password
        self.mobilebackup2_send_request(
            "Restore", self.udid, self.udid, options)
        self.work_loop()

    def info(self, options={}):
        self.mobilebackup2_send_request("Info", self.udid, options)
        info = self.work_loop()
        if info:
            pprint(info.get("Content"))
        return info

    def list(self, options={}):
        self.mobilebackup2_send_request("List", self.udid, options)
        z = self.work_loop()
        if z:
            print(z["Content"])
        return z

    def changepw(self, oldpw, newpw):
        options = {"OldPassword": oldpw,
                   "NewPassword": newpw}

        self.mobilebackup2_send_request("ChangePassword", self.udid, "")
        z = self.work_loop()
        if z:
            print(z)
        return z

    def unback(self, options={"Password": None}):
        self.mobilebackup2_send_request("Unback", self.udid, options)
        print(self.work_loop())

    def enableCloudBackup(self, options={"CloudBackupState": False}):
        self.mobilebackup2_send_request(
            "EnableCloudBackup", self.udid, options)
        print(self.work_loop())


if __name__ == "__main__":
    parser = OptionParser(usage="%prog")
    parser.add_option("-b", "--backup", dest="backup", action="store_true", default=True,
                      help="Backup device")
    parser.add_option("-r", "--restore", dest="restore", action="store_true", default=False,
                      help="Restore device")
    parser.add_option("-i", "--info", dest="info", action="store_true", default=False,
                      help="Show backup info")
    parser.add_option("-l", "--list", dest="list", action="store_true", default=False,
                      help="Show backup info")
    (options, args) = parser.parse_args()

    lockdown = LockdownClient()
    mb = MobileBackup2(lockdown)

    if options.backup:
        mb.backup()
    elif options.restore:
        mb.restore()
    if options.info:
        mb.info()
    if options.list:
        mb.list()
